function Data=MCpiMast(C,N)
% MAST: Master for computing pi using MPITB
%
%	Data = Mast ( C, N)
%
%  C	#computers used (NOT including this one)
%  N	#samples for aggregate total number of random number
%  Communication primitive: send 
%
%  Data	structure contains:
%	pi	estimated pi value
%	err	error
%	time	from argument xmit to pi computed
%
% pi is obtained by Monte-Carlo simulation.  Number of random number
% fall in first quadrant of circle with unit radius.
% MATLAB has a pi function, that we use to estimate error
%
% lamboot should have booted at least C+1 computers
%

Data=[]; TAG=7; 
DEBUG=0;   % 1 for debugging purpose
%%%%%%%%%%
% ArgChk %
%%%%%%%%%%
if nargin<2, help(mfilename), return, end
flag=0;                                 CC=int2str(C);
flag=flag | fix(C)~=C | C<0;            NN=int2str(N);
flag=flag | fix(N)~=N | N<1;            
if flag, help(mfilename), return, end

if C
%%%%%%%%%%%%%%%%%%%
% Test daemon LAM % # hosts > C
%%%%%%%%%%%%%%%%%%%
  [info flag]=MPI_Initialized;
  if info | ~flag
    RTFLAGS=RTF_HOMOG+RTF_MPIC2C*1;
    putenv(['TROLLIUSRTF=' int2str(RTFLAGS)]), MPI_Init;
  end
  RTFLAGS=str2num(getenv('TROLLIUSRTF'));
  if ~bitand(RTFLAGS,RTF_HOMOG)
	warning('use mpirun -O for homogeneous cluster!!!');
  end
  [info attr flag]=MPI_Attr_get(MPI_COMM_WORLD,MPI_UNIVERSE_SIZE);
  if info | ~flag, error('attribute MPI_UNIVERSE_SIZE does not exist?'), end
  if attr<=C, error(['need ' int2str(C+1) ' computers in LAM']), end

%%%%%%%%%
% SPAWN %
%%%%%%%%%
  if DEBUG
    DISP=getenv('DISPLAY');
    [info children errs]=MPI_Comm_spawn('/usr/X11R6/bin/xterm',...
		{'-display' DISP '-e' 'matlabr12'},C,'NULL',0,'SELF');
  else
    [info children errs]=MPI_Comm_spawn('matlabr12',...
				  {'-nosplash'},C,'NULL',0,'SELF');
  end
  if info | any(errs),	error('cannot start MATLABs'), end
  [info numt] = MPI_Comm_remote_size(children);
  if info | numt~=C,	error('cannot start MATLABs'), end
  [info NEWORLD]=MPI_Intercomm_merge(children,0);
  if info,		error('cannot start MATLABs'), end

  if DEBUG
    MPI_Errhandler_set(NEWORLD,MPI_ERRORS_RETURN);
  end

%%%%%%%%%%%
% COMMAND %
%%%%%%%%%%%
%%%%%%%%%%%%
% PARALLEL %
%%%%%%%%%%%%
    cmd=['MCpiWork(' CC ',' NN ')'];
    MPI_Bcast(cmd,0,NEWORLD);		% Pass command
    MPI_Barrier(NEWORLD);		% initial Synch
    NULL=[];
    for numt=1:C,  MPI_Recv(NULL,MPI_ANY_SOURCE,TAG,NEWORLD); end
    for numt=1:C,  MPI_Send(NULL,numt,          TAG,NEWORLD); end
%%%%%%%%%
    tic %
%%%%%%%%%
    P2=0; Psum=0;
    for numt=1:C,  MPI_Recv(P2,MPI_ANY_SOURCE,TAG,NEWORLD); Psum=Psum+P2; end

    Psum     =4*Psum/N;		% better done in master: don't lose resolution
    Data.pi  =Psum;
    Data.err =sqrt(Psum*(4-Psum)/N);
%    Data.err =Psum-pi;
%%%%%%%%%%%%%%%%%%%
    Data.time=toc;%
%%%%%%%%%%%%%%%%%%%
%   MPI_Finalize;		% No, no, perhaps we want to go on
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

else % if C
%%%%%%%%%%%%%%
% SEQUENTIAL %
%%%%%%%%%%%%%%
  tic %
%%%%%%%
f=inline('x.^2+y.^2','x' ,'y');
rank=1;
rand('state',sum(100*clock)+rank*N)
x=rand(N,1); y=rand(N,1);
Sum=4/N*sum(f(x,y)<=1,1);
  Sum      =Sum;
  Data.pi  =Sum;
  Data.err =sqrt(Sum*(4-Sum)/N);
%  Data.err =Sum-pi;
%%%%%%%%%%%%%%%%%
  Data.time=toc;%
%%%%%%%%%%%%%%%%%

end % if C

% if abs(Data.err)>5E-10, warning('pretty nice pi value!'), end

